[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# FactSet Mergers and Acquisitions client library for Python

[![API Version](https://img.shields.io/badge/api-v1.0.0-blue)](https://developer.factset.com/api-catalog/factset-mergers-acquisitions-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.FactSetMergersandAcquisitions)](https://pypi.org/project/fds.sdk.FactSetMergersandAcquisitions/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

The FactSet Mergers and Acquisitions API provides access to key transactions and pricing metrics on over 60,000 globally traded deals. Reference a Global Universe of Mergers and Acquisitions (M&A) transactions where the target of the deal is a Publicly traded Company. Review premiums, multiples and fundamental data to gather insights on deals and create predictive models based on different deal characteristics.

  Monetary values returned by this API are converted and represented in USD.

  This API is allows a limit of 10 requests per second and a maximum of 10 concurrent requests.


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.0.0
- SDK version: 1.0.4
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.FactSetMergersandAcquisitions==1.0.4
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.FactSetMergersandAcquisitions==1.0.4
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.FactSetMergersandAcquisitions
from fds.sdk.FactSetMergersandAcquisitions.api import deals_api
from fds.sdk.FactSetMergersandAcquisitions.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.FactSetMergersandAcquisitions.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.FactSetMergersandAcquisitions.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.FactSetMergersandAcquisitions.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = deals_api.DealsApi(api_client)
    deals_request = DealsRequest(
        data=DealsRequestBody(
            ids=Ids(["FDS-US"]),
            start_date="2017-12-01",
            end_date="2017-12-31",
        ),
    ) # DealsRequest | Request object for `Deals`.

    try:
        # Returns details for a list of companies.
        # example passing only required values which don't have defaults set
        api_response = api_instance.get_deals_by_company_list(deals_request)

        pprint(api_response)
    except fds.sdk.FactSetMergersandAcquisitions.ApiException as e:
        print("Exception when calling DealsApi->get_deals_by_company_list: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Returns details for a list of companies.
    #     api_response, http_status_code, response_headers = api_instance.get_deals_by_company_list_with_http_info(deals_request)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetMergersandAcquisitions.ApiException as e:
    #     print("Exception when calling DealsApi->get_deals_by_company_list: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Returns details for a list of companies.
    #     async_result = api_instance.get_deals_by_company_list_async(deals_request)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.FactSetMergersandAcquisitions.ApiException as e:
    #     print("Exception when calling DealsApi->get_deals_by_company_list: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Returns details for a list of companies.
    #     async_result = api_instance.get_deals_by_company_list_with_http_info_async(deals_request)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetMergersandAcquisitions.ApiException as e:
    #     print("Exception when calling DealsApi->get_deals_by_company_list: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.FactSetMergersandAcquisitions

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.FactSetMergersandAcquisitions.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.FactSetMergersandAcquisitions

configuration = fds.sdk.FactSetMergersandAcquisitions.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.FactSetMergersandAcquisitions

configuration = fds.sdk.FactSetMergersandAcquisitions.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.FactSetMergersandAcquisitions

configuration = fds.sdk.FactSetMergersandAcquisitions.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/content/factset-mergers-acquisitions/v1*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*DealsApi* | [**get_deals_by_company_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealsApi.md#get_deals_by_company_list) | **POST** /deals/by-company | Returns details for a list of companies.
*DealsApi* | [**get_deals_details_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealsApi.md#get_deals_details_list) | **POST** /deals/details | Returns details for specified deals.
*DealsApi* | [**get_public_target_deals_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealsApi.md#get_public_target_deals_list) | **POST** /deals/public-targets | Returns a list of deals in which the target is a public company.


## Documentation For Models

 - [DealCompany](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealCompany.md)
 - [DealIds](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealIds.md)
 - [DealPublic](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealPublic.md)
 - [DealValue](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealValue.md)
 - [DealsPublicRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealsPublicRequest.md)
 - [DealsPublicRequestBody](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealsPublicRequestBody.md)
 - [DealsPublicResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealsPublicResponse.md)
 - [DealsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealsRequest.md)
 - [DealsRequestBody](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealsRequestBody.md)
 - [DealsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DealsResponse.md)
 - [Detail](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/Detail.md)
 - [DetailsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DetailsRequest.md)
 - [DetailsRequestBody](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DetailsRequestBody.md)
 - [DetailsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/DetailsResponse.md)
 - [ErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/ErrorObject.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/ErrorResponse.md)
 - [Ids](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/Ids.md)
 - [Participant](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/Participant.md)
 - [Status](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/Status.md)
 - [Target](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetMergersandAcquisitions/v1/docs/Target.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.FactSetMergersandAcquisitions.apis and fds.sdk.FactSetMergersandAcquisitions.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.FactSetMergersandAcquisitions.api.default_api import DefaultApi`
- `from fds.sdk.FactSetMergersandAcquisitions.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.FactSetMergersandAcquisitions
from fds.sdk.FactSetMergersandAcquisitions.apis import *
from fds.sdk.FactSetMergersandAcquisitions.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

